#
# -*- coding: utf-8 -*-

"""
    sort_order
    ~~~~~~~~~~

    The Sphinx extension to provide sort order.

    :copyright: (c) 2011-2013, 2015 by Suzumizaki-Kimitaka.
    :license: 2-clause BSD.
"""

"""
This file is imported by 'user_ordered_index_patch.py', which changes
the order of the items in the indices.

'yogosyu.py' also imports this file. That extends glossary directive.

Both of '.py' file above are the extensions for the Sphinx.
To enable these feature, make your 'ext' setting in 'conf.py' to import
'yogosyu.py' and 'user_ordered_index_patch.py'.

On the other hand, this module itself is NOT extension.
Just for subclassing or helper functions of another extensions.

You can define 'sort_order_LANG.py' to give new order for indices,
instead of the unicode codepoint order.

Any 'sort_order_LANG.py' should have the function named 
'get_default_sort_order(cfg)'. The function should return the object
which inherits 'SortOrderBase' defined in this file.

Updated at Mar 2015, for Python 3.4.3 and Sphinx 1.3
"""

import unicodedata
import string
import importlib

class SortOrderBase(object):
    """Base class of any SortOrder-s

    Any sort order classes should inherit this class.
    'SortOrderLegacy' is defined in this file, 'sort_order.py'.
    You can refer 'sort_order_ja.py' as the sample implementation.
    You may want to inherit 'SortOrderLegacy' instead of this class.
    See the definition below.
    """

    def get_string_to_sort(self, entry_name, yomi_dict = {}):
        """Return the string for sorting instead of given name

        @param name the name of the entry
        @param yomi_dict name to yomigana(user sort key) dictionary
        @return the internal sort key, should be sortable each other

        All classes inherit this class may override this function.

        Return the internal sort key refering the given name.
        
        The internal sort key should be sortable with the unicode
        codepoint order. For example, when your indices have the
        group "Etc." or "Symbols", the return value from this function
        should have the group marker character prefix against such 
        entries.
        
        Like above, when unfortunately your grouping cannot unify 
        single range of the unicode codepoint, the return value also
        should have prefix.
        
        You can just return the entry_name if you don't have to give
        internal sort key.
        
        Check the implementation of 'SortOrderLegacy' below.
        You may want to inherit that class to ignore cases or so.
        """
        return entry_name

    def get_group_name(self, entry_name, yomi_dict = {}):
        """Return the group name of the given entry

        @param name the name of the entry
        @param yomi_dict name to yomigana(user sort key) dictionary
        @return the name of the group which the given name belongs to

        Grouping itself should be done by 'get_string_to_sort()'
        function defined above.
        See the implementation of 'SortOrderLegacy' below.
        """
        return entry_name[0]


class SortOrderLegacy(SortOrderBase):
    """Perform sort order given like Sphinx 1.0.7 or prior"""

    def get_string_to_sort(self, name, yomi_dict = {}):
        """Return the string for sorting instead of given name

        To make the grouping done, making lowercased,
        'NFD' normalization and prepend prefix to symbols.
        """
        lcletters = string.ascii_lowercase + '_'
        lckey = unicodedata.normalize('NFD', name.lower())
        if lckey[0:1] in lcletters:
            return chr(127) + lckey
        return lckey

    def get_group_name(self, entry_name, yomi_dict = {}):
        """Return the group name of the given entry

        This class/function gives just only English specific
        sort order. The alphabet is limited only in ascii and
        force all characters NFD normalization.
        """        
        letters = string.ascii_uppercase + '_'
        letter = unicodedata.normalize('NFD', entry_name)[0].upper()
        if letter in letters:
            return letter
        else:
            return 'Symbols'


class SortOrderLegacyLike(SortOrderBase):
    """Base class for lauguages similar to latin."""

    def get_string_to_sort(self, name, yomi_dict = {}):
        s = unicodedata.normalize('NFD', name).upper()
        if s[:1] not in self.get_uppercases():
            return u"\uffff" + s
        return s

    def get_group_name(self, entry_name, yomi_dict = {}):
        """Return the group name of the given entry"""
        s = self.get_string_to_sort(entry_name)
        if s[0] in self.get_uppercases():
            return s[0]
        return self.etc_form()

    def etc_form(self):
        """Return the word 'etc.' in your language

        Override this.
        """
        return u"Etc."

    def get_uppercases(self):
        """Return uppercased characters used in your language

        Override this.
        """
        return string.ascii_uppercase


def get_default_sort_order(cfg):
    """Return the default sort order of the language given in conf.py

    @param cfg app.config, builder.config, etc.
    @return the object of the class inherits SortOrderBase.

    Called by another extensions.
    See also get_sort_order() function.
    """
    try:
        lang = cfg.language
    except:
        lang = None
    if not lang:
        e = 'sort_order.py: ' + \
            'No SortOrderBase object applied,' +  \
            ' and no language assigned.' +  \
            ' using SortOrderLegacy.'
        print(e)
        return SortOrderLegacy()
    import_target = 'sort_order_' + lang
    try:
        mod = importlib.import_module(import_target)
        return mod.get_default_sort_order(cfg)
    except ImportError:
        e = 'sort_order.py: ' + \
            'No SortOrderBase object applied,' + \
            ' and module {0} not found.' + \
            ' using SortOrderLegacy.'
        print(e.format(import_target))
        return SortOrderLegacy()


def get_sort_order(cfg):
    """Return sort order object with current configure

    @param cfg app.config, builder.config, etc.
    @return the object of the class inherits SortOrderBase.

    More convenient function than get_default_sort_order().
    Called by another extensions.
    """
    try:
        r = cfg.sort_order
        try:
            assert isinstance(r, SortOrderBase)
            return r
        except:
           print('get_sort_order.py: ' + \
                 'config "sort_order" should be' + \
                 'the object of SortOrderBase.')
    except:
        pass
    return get_default_sort_order(cfg)


def setup(app):
    """Called by the Sphinx to extend it.

    @param app 'sphinx.application.Sphinx' object to use add builder or so.
    
    In this module, this function is just a place holder. Do nothing.
    """
    return
